import logger from '../../../middleware/logger';

const fetch = require('node-fetch');

export const getiCalFile = async (user: string, file: string, showBlockers: boolean, electiveModule: string, profileModule: string): Promise<string> => {
    try {
        let iCalFileContents = '';

        let baseUrl = 'https://rapla.dhbw-karlsruhe.de/rapla?page=ical';
        baseUrl += '&user=' + user;
        baseUrl += '&file=' + file;

        const response = await fetch(baseUrl);

        let body = await response.text();

        // Parse the ical file
        let iCalFile = parseIcal(body);

        // Now remove unwanted shit from the iCal file
        if (!showBlockers) {
            iCalFile = removeBlockers(iCalFile);
        }

        if (electiveModule !== '') {
            iCalFile = removeElective(iCalFile, electiveModule);
        }

        if (profileModule !== '') {
            iCalFile = removeProfile(iCalFile, profileModule);
        }

        // Turn into one file again
        let resultingFile = serializeIcal(iCalFile);

        return resultingFile;
    } catch (err) {
        throw err;
    }
};

export const parseIcal = function (icalContents: string): iCalFile {
    const fileHeaderRegex: RegExp = /^BEGIN:VCALENDAR.*?BEGIN:VEVENT$/ms;
    const eventRegex: RegExp = /^BEGIN:VEVENT.*?END:VEVENT$/msg;

    let headerMatch = fileHeaderRegex.exec(icalContents);
    let header = '';

    if (headerMatch) {
        header = headerMatch[0];
    }

    header = header.substr(0, header.lastIndexOf('\n'));

    let events: string[] = [];
    let match: any;
    while (match = eventRegex.exec(icalContents)) {
        if (match) {
            events.push(match[0]);
        }
    }

    return {
        head: header,
        body: events
    };
};

export const serializeIcal = function (ical: iCalFile): string {
    let resString = ical.head;
    ical.body.forEach((event) => {
        resString += event + '\n';
    });
    resString += 'END:VCALENDAR';

    return resString;
};

export const removeBlockers = function (ical: iCalFile): iCalFile {
    let remainingEvents: string[] = [];

    ical.body.forEach((event) => {
        if (!event.includes('CATEGORIES:Sonstiger Termin') && !event.includes('SUMMARY:Präsenz')) {
            remainingEvents.push(event);
        }
    });

    ical.body = remainingEvents;

    return ical;
};

export const removeElective = function (ical: iCalFile, chosenElective: string): iCalFile {
    let remainingEvents: string[] = [];

    let electiveToRemove = electiveModules;
    electiveToRemove.splice(parseInt(chosenElective), 1);

    ical.body.forEach((event) => {
        let addEvent = true;
        electiveToRemove.forEach((module) => {
            if (event.includes(module.name)) {
                logger.info('Removing elective ' + module.name);
                addEvent = false;
            }
        });

        if (addEvent) {
            remainingEvents.push(event);
        }
    });

    ical.body = remainingEvents;

    return ical;
};

export const removeProfile = function (ical: iCalFile, chosenProfile: string): iCalFile {
    let remainingEvents: string[] = [];

    let profileToRemove = profileModules;
    profileToRemove.splice(parseInt(chosenProfile), 1);

    ical.body.forEach((event) => {
        let addEvent = true;
        profileToRemove.forEach((module) => {
            module.names.forEach((name) => {
                if (event.includes(name)) {
                    logger.info('Removing profile ' + name);
                    addEvent = false;
                }
            });
        });

        if (addEvent) {
            remainingEvents.push(event);
        }
    });

    ical.body = remainingEvents;

    return ical;
};

export interface iCalFile {
    head: string;
    body: string[];
}

let electiveModules = [
    {name: 'ERP-Systeme'},
    {name: 'Ethik für Informatiker'},
    {name: 'Evolutionäre Algorithmen'},
    {name: 'Forensik'},
    {name: 'Kryptographische Verfahren'},
    {name: 'Robotik'},
    {name: 'Web-Services'}
];

let profileModules = [
    {names: ['.Grundlagen der KI (KI und BV)', 'Bildverarbeitung (KI und BV)']},
    {names: ['Bildverarbeitung', 'Computergraphik']},
    {names: ['Interaktive Systeme', 'Grundlagen der KI']},
    {names: ['e Business', 'angewandtes Projektmanagement']},
    {names: ['Kommunikations und Netztechnik II']}
];
